package org.java.evolutionary.sequence;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Iterator;

import org.biojava.bio.seq.DNATools;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.SequenceIterator;
import org.biojava.bio.seq.io.SeqIOTools;
import org.biojava.bio.symbol.FiniteAlphabet;
import org.biojava.bio.symbol.SymbolList;
import org.biojava.utils.regex.Matcher;
import org.biojava.utils.regex.Pattern;
import org.biojava.utils.regex.PatternFactory;

import ec.EvolutionState;
import ec.Individual;
import ec.Problem;
import ec.gp.GPIndividual;
import ec.gp.GPProblem;
import ec.gp.koza.KozaFitness;
import ec.simple.SimpleFitness;
import ec.simple.SimpleProblemForm;
import ec.util.Parameter;

public class AluRecognitionProblem extends GPProblem implements
		SimpleProblemForm, SequenceClassificationProblem {
	protected static double positiveSequences;
	protected static double negativeSequences;
	protected static ArrayList<SymbolList> positiveSequencesList = new ArrayList<SymbolList>();
	protected static ArrayList<SymbolList> negativeSequencesList = new ArrayList<SymbolList>();
	private PatternFactory[] factory;

	public static final String POSITIVE_FILE_NAME_PARAM = "positivefile";
	public static final String NEGATIVE_FILE_NAME_PARAM = "negativefile";
	
	protected SymbolList currentSequence;
	
	 /** The small deviation allowed in double comparisons. */
	  public static double SMALL = 1e-9;
	  
	  public static double MEDIUM = 1e-5;

	

	public void setup(final EvolutionState state, final Parameter base) {
		// make sure parents are ok
		super.setup(state, base);
		// get positive file name
		String positiveFileName = state.parameters.getString(base
				.push(POSITIVE_FILE_NAME_PARAM), null);
		// get negative file name
		String negativeFileName = state.parameters.getString(base
				.push(NEGATIVE_FILE_NAME_PARAM), null);
		BufferedReader positiveBuffer;
		BufferedReader negativeBuffer;
		try {
			// read the files
			positiveBuffer = new BufferedReader(
					new FileReader(positiveFileName));

			
			SequenceIterator positiveFileIterator = (SequenceIterator) SeqIOTools
					.fileToBiojava("fasta", "dna", positiveBuffer);
		
			positiveSequences = this.countTotalSequence(positiveFileIterator,
					true);
			
			// get positive file name
			negativeFileName = state.parameters.getString(base
					.push(NEGATIVE_FILE_NAME_PARAM), null);
			BufferedReader reader = new BufferedReader(new InputStreamReader(
						new FileInputStream(negativeFileName)));

				String totalLine = reader.readLine();
				
				int i = 0;
				while (totalLine != null)
	            {
	                String sequence, label;
	                int startOfSpace = totalLine.indexOf(" ");
	                String token1 = totalLine.substring(0, startOfSpace);
	                String token2 = totalLine.substring(startOfSpace + 1, totalLine.length());
	                if (token1.equals("1") || token1.equals("-1"))
	                     { label = token1; sequence = token2; }
	                else { label = token2; sequence = token1; }

					if (label.equals("-1"))
						negativeSequencesList.add(DNATools.createDNA(sequence));
					totalLine = reader.readLine();
					System.out.println("Completed:" + i++);
				}
			
			this.negativeSequences = this.negativeSequencesList.size();
			System.out.println("Positive number:" + this.positiveSequences
					+ " Negative number:" + this.negativeSequences);

		} catch (Exception e) {
			e.printStackTrace();
		}

		FiniteAlphabet iupac = DNATools.getDNA();
//		FiniteAlphabet amino = ProteinTools.getAlphabet();
		factory = new PatternFactory[4];
		for(int i=0; i<4; i++)
			factory[i] = PatternFactory.makeFactory(iupac);
	}
	
	public SymbolList getCurrentSequence(){
		return this.currentSequence;
	}

	private int countTotalSequence(SequenceIterator iterator, boolean positive) {
		int total = 0;
		try {
			while (iterator.hasNext()) {
				Sequence seq = iterator.nextSequence();
				total = total + 1;
				if (positive)
					this.positiveSequencesList.add(seq);
				else
					this.negativeSequencesList.add(seq);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return total;
	}

	public void evaluate(final EvolutionState state, final Individual ind,
			final int subpopulation, final int threadnum) {
		if (!ind.evaluated)  // don't bother reevaluating
        {
			int truePositiveCount =0;
			int trueNegativeCount =0;
			
			//iterate positive sequences 
			for(int i=0; i< positiveSequencesList.size(); i++){
				this.currentSequence = positiveSequencesList.get(i);
				
				MotifData motifData = new MotifData();
				//now evaluate the problem on GP
				((GPIndividual)ind).trees[0].child.eval(
	                    state,threadnum, motifData,stack,((GPIndividual)ind),this);
				if(motifData.matches){
					truePositiveCount = truePositiveCount +1;
				}
			}
			//iterate over negative sequences
			for(int i=0; i< negativeSequencesList.size(); i++){
				this.currentSequence = negativeSequencesList.get(i);
				MotifData motifData = new MotifData();
				//now evaluate the problem on GP
				((GPIndividual)ind).trees[0].child.eval(
	                    state,threadnum, motifData,stack,((GPIndividual)ind),this);
				if(motifData.matches){
					trueNegativeCount = trueNegativeCount +1;
				}
			}
			
//			ind.printIndividualForHumans(state,0,1000);
//			StringWriter writer = new StringWriter();
//			PrintWriter printWriter = new PrintWriter(writer);
//			ind.printIndividual(state, printWriter);
//			printWriter.flush();
//			writer.flush();
//			String representation = writer.toString();
//			System.out.println("True Positive count: " + truePositiveCount +  " and True Negative Count: " + trueNegativeCount +" for " + representation);
			double precision = 0;
			precision = truePositiveCount/positiveSequences;
		    double negative = trueNegativeCount/negativeSequences;
		    double informationGain = Math.abs(precision-negative);		
			double combined = SMALL;
			boolean dontWrite = false;
//			if(!representation.contains("matches")  && !representation.contains("present")){
//				dontWrite = true;
//			}						
			//fitness is precision
			//else if((representation.contains("matches") && truePositiveCount >0) || (representation.contains("present") && truePositiveCount >0)){
//			else{
//			    precision = truePositiveCount/positiveSequences;
//			    double[][] counts = new double[2][2];
//			    counts[0][0] = positiveSequences-truePositiveCount;
//			    counts[0][1] = negativeSequences-trueNegativeCount;
//			    counts[1][0] = truePositiveCount;
//			    counts[1][1]=  trueNegativeCount;
//			    //caclulate information gain
//			    informationGain = Math.abs(entropyConditionedOnColumns(counts)- entropyConditionedOnRows(counts));
//			    System.out.println("InformationGain: " + informationGain);
			    if(informationGain >0 && precision > 0)
			    	combined =  precision * informationGain;
//			}
			
			
			
			//form koza fitness
			KozaFitness f = ((KozaFitness)ind.fitness);
			//to avoid ideal individual
			//if(precision == 1.0)
			//	precision = 0.9;
			f.setStandardizedFitness(state,(float)(1.0/combined)); 
			//set hits
			f.hits = positiveSequencesList.size();
			//set evaluated
			ind.evaluated = true;   			
//			if(truePositiveCount > 3 && !dontWrite && combined > SMALL)
//				FeatureCollector.writeFeature((GPIndividual)ind, state);
			
//			System.out.println("Finished evaulating fitness:" + combined);
        }
	}
	
	public PatternFactory getFactory(int thread){
		return factory[thread];
	}
	
	
}
