package org.java.evolutionary.sequence.func;

import org.biojava.bio.symbol.SymbolList;
import org.biojava.utils.regex.Matcher;
import org.biojava.utils.regex.Pattern;
import org.biojava.utils.regex.PatternFactory;

import org.java.evolutionary.sequence.MotifData;
import ec.EvolutionState;
import ec.Problem;
import org.java.evolutionary.sequence.SequenceClassificationProblem;
import ec.gp.ADFStack;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.util.Parameter;

public class ShiftPositional extends GPNode{
	public String toString() {
		return " shiftPosition ";
	}

	public void checkConstraints(final EvolutionState state, final int tree,
			final GPIndividual typicalIndividual, final Parameter individualBase) {
		super.checkConstraints(state, tree, typicalIndividual, individualBase);
		if (children.length != 3)
			state.output.error("Incorrect number of children for node "
					+ toStringForError() + " at " + individualBase);
	}

	public void eval(final EvolutionState state, final int thread,
			final GPData input, final ADFStack stack,
			final GPIndividual individual, final Problem problem) {
		MotifData motifData = ((MotifData) (input));
		//child should give me motif
		children[0].eval(state, thread, input, stack, individual, problem);	
		//child should give me position
		children[1].eval(state, thread, input, stack, individual, problem);	
		//child should give me proximity
		children[2].eval(state, thread, input, stack, individual, problem);		
		//get the curernt motif
		char[] currentMotif = motifData.currentMotif;
		//get the current position
		int position = motifData.currentPosition;
		//proximity
		int proximity = motifData.close;
		//check for match using biojava
		SequenceClassificationProblem spliceSetProblem =(SequenceClassificationProblem)problem;
		SymbolList sequence = spliceSetProblem.getCurrentSequence();
		PatternFactory factory = spliceSetProblem.getFactory(thread);
		motifData.matches = matches(sequence,currentMotif,position,proximity, factory);
	}
	
	private boolean matches(SymbolList seq, char[] currentMotif, int position,int proximity, PatternFactory factory){
		boolean matched = false;
		try{
			//in the sequence find the match
			Pattern p = factory.compile(new String(currentMotif));
			Matcher occurences = p.matcher(seq);
			while (occurences.find()) {
				int current = occurences.start();
				if (Math.abs(position-current) <= proximity) {
					matched = true;
					break;
				}
			}
		}catch(Exception e){
			System.out.println("Exception at Motif: " + new String(currentMotif) + " position:" + position + " seq:" + seq.seqString());
			e.printStackTrace();
		}
		return matched;
	}
}
