package org.java.featurebased.gibbs;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;

import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.SequenceIterator;
import org.biojava.bio.seq.io.SeqIOTools;
import org.java.featurebased.AbstractFeatureBased;
import org.java.featurebased.FeatureBasedMotif;

public class GibbSamplingMotifFeatureGenerator extends AbstractFeatureBased{
	

	boolean completed = false;
	
	public GibbSamplingMotifFeatureGenerator(String positiveFile, boolean fastaPositive,String negativeFile, boolean fastaNegative, String outputFile, int motifLength){
		super(positiveFile,fastaPositive,negativeFile,fastaNegative,outputFile,motifLength);
	}
	
	public void generateFeatures() throws Exception{
		if(!completed){
			//use Gibbs sampling to generate motifs from 2 to k
			//a file of sequences sequences
		    File f = new File(positiveFile);
		    
		    for(int i =2; i<= this.motifLength; i++){
		    	BufferedReader br = new BufferedReader(new FileReader(f));
			    SequenceIterator it =(SequenceIterator)SeqIOTools.fileToBiojava("fasta", "DNA", br);
		    	SimpleGibbsAligner gibbs = new SimpleGibbsAligner(i,
		            it, GibbsStoppingCriteria.HEURISTIC);
		        //start the aligner running
		        gibbs.iterate();
		        //get the sequences, offsets and window size to print out the motif
		        Sequence[] seqs = gibbs.getSequences();
		        int[] offSets = gibbs.getOffSets();
		        int wind = gibbs.getWindowSize();
		   
		        //print out the motif
		        for (int j = 0; j < offSets.length; j++) {
	//	          System.out.println(seqs[j].subStr(offSets[j],offSets[j]+wind -1));
		          features.add(seqs[j].subStr(offSets[j],offSets[j]+wind -1));
		        }
		        System.out.println("Current Size of Features: " + features.size());
		    }
		    completed=true;
		}
		
	}
	
	public static void main(String[] args) throws Exception{
		if(args.length <4)
			throw new IllegalArgumentException("java GibbSamplingMotifFeatureGenerator posFile negFile outputFile KmerLength ");
		AbstractFeatureBased f = new GibbSamplingMotifFeatureGenerator(args[0], true, args[1], true, args[2], new Integer(args[3]).intValue());
		f.generateLibSVMFile();
		if(args.length == 7){
		//replace train with test 
			f.changeForTest(args[4], args[5], args[6]);
			f.generateLibSVMFile();
		}
	}

}
